@@ -11,12 +11,12 @@ import datetime as dt
 
 class WikiScraper:
 
-    def __init__(self,base_url="https://en.wikipedia.org"):
+    def __init__(self,base_url="https://en.wikipedia.org",headers={"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36"},):
         self.base_url = base_url
 
 
     @property
-    def disney_movies_urls(self):
+    def disney_movies(self):
         """
             This property returns the disney movies' urls 
             from the wikipedia page
@@ -27,7 +27,7 @@ class WikiScraper:
         tables = soup.select(".wikitable.sortable tbody")
 
         data_points = []
-        links = []
+        data_list = []
         for table in tables: 
             rows = table.find_all("tr")
             for row in rows: 
@@ -36,21 +36,33 @@ class WikiScraper:
                 except Exception as e:  
                     pass
             
-        for data in data_points: 
+        for data in data_points:
+            disney_movie = {} 
+            link_text = data.find("a")
+
+            try:
+                name = link_text.get_text("",strip=True)
+            except Exception as e:
+                pass
+            else:
+                disney_movie["name"] = name
+
             try: 
-                link = data.find("a")["href"]
+                link = link_text["href"]
             except Exception as e: 
                 pass
             else: 
                 if not(link[0] == "#"):
                     link = self.base_url + link 
-                    links.append(link)
+                    disney_movie["url"] = link
 
-        return links
+            data_list.append(disney_movie)
+
+        return data_list
 
 
     @property
-    def highest_grossing_movies_urls(self):
+    def highest_grossing_movies(self):
         """
             This function returns the urls of the highest grossing movies 
             in the world righ now.
@@ -61,19 +73,32 @@ class WikiScraper:
 
         table = soup.select_one(".wikitable.sortable").find("tbody")
 
-        highest_grossing_urls = []
+        highest_grossing_movies = []
 
         for row in table.find_all("tr"):
+
+            movie_object = {}
+            link_tag = row.find_all("th")[0].find("a")
+
             try:
-                link_tag = row.find_all("th")[0].find("a")
+                movie_name = link_tag.get_text("",strip=True)
+            except Exception as e:
+                pass 
+            else:
+                movie_object["name"] = movie_name
+
+            try:    
                 link = link_tag["href"]
+                link = self.base_url+link
             except Exception as e:
-                print(e)
                 pass
             else:
-                highest_grossing_urls.append(self.base_url+link)
+                movie_object["url"] = link
+
+            if any(movie_object):
+                highest_grossing_movies.append(movie_object)
         
-        return highest_grossing_urls
+        return highest_grossing_movies
 
     
     @property
@@ -98,12 +123,11 @@ class WikiScraper:
                 name = person_tag.get_text("",strip=True)
                 person_url = self.base_url + person_tag["href"]
             except Exception as e:
-                print(e)
                 pass
             else:
                 person_object = {
                     "name": name,
-                    "person_url": person_url,
+                    "url": person_url,
                 }
 
                 people_list.append(person_object)
@@ -112,7 +136,7 @@ class WikiScraper:
         return people_list
 
 
-    def scrape_movie_info(self,url): 
+    def scrape_movie(self,url): 
         """
             You need to pass a wikipedia movie url into this function
             and it will return the info about that movie
@@ -134,7 +158,7 @@ class WikiScraper:
         for idx, row in enumerate(info_box.find_all("tr")):
 
             if idx == 0: 
-                movie_info["title"] = row.get_text()
+                movie_info["Name"] = row.get_text()
             
             if idx == 1: 
                 continue
@@ -164,7 +188,6 @@ class WikiScraper:
             minute_unit = re.findall(minutes_unit_pattern,movie_info["Running time"])[-1]
             movie_info[f"Running time({minute_unit})"] = minutes
         except Exception as e:
-            print(e)
             pass
         else: 
             del movie_info["Running time"]
@@ -175,7 +198,6 @@ class WikiScraper:
             currency, gross = money_string_to_int(movie_info,"Box office")
             movie_info[f"Gross({currency})"] = gross
         except Exception as e:
-            print(e)
             pass
         else:
             del movie_info["Box office"]
@@ -184,7 +206,6 @@ class WikiScraper:
             currency, budget = money_string_to_int(movie_info,"Budget")
             movie_info[f"Budget({currency})"] = budget
         except Exception as e:
-            print(e)
             pass
         else:
             del movie_info["Budget"]
@@ -193,7 +214,7 @@ class WikiScraper:
         return movie_info
 
 
-    def async_scrape_movies_info(self,urls):
+    def async_scrape_movies(self,urls):
         """
             This function utlizies the 'scrape_movie_info' method 
             in multiple threads 
@@ -203,7 +224,7 @@ class WikiScraper:
 
         with concurrent.futures.ThreadPoolExecutor() as executor: 
             for url in urls:
-                future = executor.submit(self.scrape_movie_info,url)
+                future = executor.submit(self.scrape_movie,url)
                 data = future.result()
                 movies_urls.append(data)
 
@@ -218,6 +239,10 @@ class WikiScraper:
 
         put_year_limit(year,1920)
 
+        try:
+            year = int(year)
+        except:
+            raise(Exception("Please enter a valid year"))
 
         url = f"https://en.wikipedia.org/wiki/{year}_in_film"
         r = requests.get(url)
@@ -231,7 +256,7 @@ class WikiScraper:
             if ul:
                 lists.append(ul)
 
-        def scrape_movies_year_list(ul):
+        def scrape_movies_year_list(ul,movie_list):
             list_items = ul.find_all("li")
 
             for idx, item in enumerate(list_items):
@@ -243,16 +268,15 @@ class WikiScraper:
                 except Exception as e:
                     pass
                 else:
-                    movie_object["title"] = movie_title
-                    movie_object["movie_url"] = self.base_url + movie_url
-                    return movie_object
+                    movie_object["name"] = movie_title
+                    movie_object["url"] = self.base_url + movie_url
+                    if movie_object not in movie_list:
+                        movie_list.append(movie_object)
 
         movie_list = []
 
         for ul in lists: 
-            movie_data = scrape_movies_year_list(ul)
-            if movie_data and movie_data not in movie_list:
-                movie_list.append(movie_data)
+            movie_data = scrape_movies_year_list(ul,movie_list)
 
         if year >= 2004: 
 
@@ -264,12 +288,11 @@ class WikiScraper:
                     movie_url = self.base_url + movie_tag["href"]
                     movie_title = movie_tag.get_text("",strip=True)
                 except Exception as e:
-                    print(e) 
                     pass
                 else:
                     movie_object = {
-                        "title": movie_title,
-                        "movie_url": movie_url
+                        "name": movie_title,
+                        "url": movie_url
                     }
                     movie_list.append(movie_object)
                 
@@ -301,7 +324,6 @@ class WikiScraper:
                     if headers.get_text("",strip=True):
                         title_index = idx
             except Exception as e: 
-                print(e)
                 pass
 
             body = table.find("tbody")
@@ -313,12 +335,11 @@ class WikiScraper:
                     movie_url = self.base_url + movie_tag["href"]
                     movie_title = movie_tag.get_text("",strip=True)
                 except Exception as e: 
-                    print(e)
                     pass
                 else: 
                     movie_object = {
-                        "title": movie_title,
-                        "movie_url": movie_url
+                        "name": movie_title,
+                        "url": movie_url
                     }
                     movie_list.append(movie_object)
 
@@ -326,13 +347,17 @@ class WikiScraper:
         return movie_list
 
 
-    def scrape_person_info(self,url="",name=""): 
+    def scrape_person(self,url="",name=""): 
         """
             This function is similar to the scraping movie info function
             but you pass in a person page's url into it and it scrapes info 
             about that person
             infobox biography vcard
         """
+
+        if not name and not url:
+            return None
+
         if not url: 
             url_name = "_".join(name.split())
             url = self.base_url + "/wiki/" + url_name
@@ -342,7 +367,7 @@ class WikiScraper:
         info_box = soup.select_one(".infobox.vcard")
 
         if not info_box: 
-            print(f"{url} table not found")
+            print(f"{url} data not found")
             return
 
         # removing all the suffix and sup element from the DOM
@@ -383,9 +408,9 @@ class WikiScraper:
                 except Exception as e: 
                     header = "Miscellaneous"
                 finally:
-                    items = [li.get_text(" ",strip=True).replace("\xa0"," ").replace("\u200b","") for li in row.find("ul").find_all("li")]
-
-                person_info[header] = items
+                    items = [li.get_text(" ",strip=True).replace("\xa0"," ").replace("\u200b","") for li in row.find("ul").find_all("li") if li.get_text(" ",strip=True)]
+                if items:
+                    person_info[header] = items
 
             # if there isn't any list scrape get items single 
             else: 
@@ -395,7 +420,13 @@ class WikiScraper:
                 except Exception as e: 
                     pass
                 else: 
-                    person_info[header] = data
+                    if data:
+                        person_info[header] = data
+
+
+
+        """Now let's get to convert and clean up the data a little"""
+
 
         # removing spaces in website url
         if "Website" in person_info.keys():
@@ -405,7 +436,6 @@ class WikiScraper:
         try: 
             del person_info["Signature"]
         except Exception as e: 
-            print(e)
             pass            
 
         # removing sports years team data
@@ -413,7 +443,6 @@ class WikiScraper:
             if person_info["Years"] == "Team":
                 del person_info["Years"]
         except Exception as e:
-            print(e)
             pass
 
         # removing awards if it is full list
@@ -421,19 +450,16 @@ class WikiScraper:
             if person_info["Awards"] == "Full list":
                 del person_info["Awards"]
         except Exception as e:
-            print(e)
             pass
                         
 
         try: 
             age = get_age(person_info,"Born")
         except Exception as e: 
-            print(e)
             try:
                 age = get_age("Date of birth")
                 person_info["Age"] = age
             except Exception as e: 
-                print(e)
                 pass
         else: 
             person_info["Age"] = age
@@ -442,14 +468,12 @@ class WikiScraper:
         try: 
             bdate = get_bdate(person_info,"Born")
         except Exception as e: 
-            print("born2 |",e)
             try: 
                 person_info["Date of birth"] = person_info["Date of birth"].strip()
                 if not person_info["Date of birth"]:
                     del person_info["Date of birth"]
 
             except Exception as e:
-                print("born2.2 |",e)
                 pass
         else: 
             person_info["Birth Date"] = bdate
@@ -458,9 +482,9 @@ class WikiScraper:
         
         # children number converted to an integer
         try: 
-            person_info["Children"] = int(person_info["Children"])
+            children_count = re.findall("\d+",person_info["Children"])[0]
+            person_info["Children"] = int(children_count)
         except Exception as e: 
-            print("children |",e)
             pass
 
 
@@ -497,10 +521,10 @@ class WikiScraper:
                 person_info["Spouse(s)"] = person_info["Spouse(s)"].strip()
                 spouse_list = person_info["Spouse(s)"].split("   ")
         except Exception as e: 
-            print("spouses |",e)
             pass
         else: 
-            person_info["Spouse(s)"] = spouse_list
+            if spouse_list:
+                person_info["Spouse(s)"] = spouse_list
 
         
         # cleaning the single data spouse
@@ -516,7 +540,6 @@ class WikiScraper:
                     present = "-present"
                 person_info["Spouse"] = person_info["Spouse"].replace(result,"("+interval+present+")")   
         except Exception as e: 
-            print("spouse |",e)
             pass
         else:
             person_info["Spouse"] = person_info["Spouse"].strip()
@@ -533,7 +556,6 @@ class WikiScraper:
                 elif "million" in person_info["Net worth"]: 
                     person_info["Net worth("+currency+")"] = 1_000_000 * float(results[0])
         except Exception as e:
-            print("net worth |",e)
             pass
         else: 
             del person_info["Net worth"]
@@ -542,47 +564,131 @@ class WikiScraper:
         return person_info
 
 
-    def async_scrape_people_info(self,urls="",names=""):
+    def async_scrape_people(self,urls="",names=""):
         """
             This function scrapes each person's info by threading. 
             It is much more efficient than running 'scrape_movie_info'
             method synchronously
         """ 
 
-        if type(urls) == dict:
-            urls = [person["person_url"] for person in urls]
-
         people_info = []
 
         if names: 
             with concurrent.futures.ThreadPoolExecutor() as executor:
                 for name in names:
-                    future = executor.submit(self.scrape_person_info,name=name)
+                    future = executor.submit(self.scrape_person,name=name)
                     person_info = future.result()
                     people_info.append(person_info)
         else: 
+            if not urls:
+                return None
             with concurrent.futures.ThreadPoolExecutor() as executor:
                 for url in urls:
-                    future = executor.submit(self.scrape_person_info,url=url)
+                    future = executor.submit(self.scrape_person,url=url)
                     person_info = future.result()
                     people_info.append(person_info)
 
         return people_info
 
 
-    def scrape_movie_cast(self,url):
-        
+    def scrape_custom(self,url="",name=""):
+        """
+            This method is a vague one, you can pass in the url of an event, company, or even a place. This function will return the data on the info box if present, otherwise it will return None 
+        """
+
+        if not url and not name:
+            return None
+
+        if not url:
+            url = "https://en.wikipedia.org/wiki/" + name
         r = requests.get(url)
         soup = BeautifulSoup(r.content, "html.parser")
 
-        cast_header = soup.find(id="Cast")
+        info_box = soup.select_one(".infobox")
 
-        cast_list = cast_header.find_next("ul")
+        if not info_box: 
+            return
 
-        cast_items = [li.find("a") for li in cast_list.find_all("li")]
-        cast_urls = [self.base_url+a["href"] for a in cast_items]
+        for sup in info_box.find_all("sup"):
+            sup.decompose()
+
+        for br in info_box.find_all("br"):
+            soup.i.string = " "
+            br.replace_with(soup.i)
+
+        for hidden in soup.find_all(style="display:none"): 
+            hidden.decompose()
+
+        custom_info = {}
+
+        caption = info_box.find("caption")
+        if caption: 
+            custom_info["Name"] = caption.get_text(" ",strip=True).replace("\xa0", " ")
+
+
+        for idx, row in enumerate(info_box.find_all("tr")):
+
+            if idx == 0 and "Name" not in custom_info.keys(): 
+                custom_info["Name"] = row.get_text("",strip=True).replace("\xa0", " ")
+            
+            if row.find("td") and row.find("ul"):
+                try: 
+                    header = row.find("th").get_text(" ", strip=True).replace("\xa0"," ")
+                    items = [li.get_text(" ",strip=True).replace("\xa0"," ") for li in row.find("ul").find_all("li")]
+                except Exception as e:
+                    pass
+                else:
+                    custom_info[header] = items
+
+            else: 
+                try: 
+                    go_to_url = row.select_one(".url")
+                    header = row.find("th").get_text(" ", strip=True).replace("\xa0"," ").replace("•","").strip()
+                    data = row.find("td").get_text(" ", strip=True).replace("\xa0"," ").replace("\ufeff", "").replace("•","").strip()
+                    if go_to_url:
+                        data = go_to_url.find("a")["href"]
+                        
+                except Exception as e: 
+                    pass
+                else: 
+                    custom_info[header] = data
+
+
+        # converting running time to integers
+        try: 
+            minute_number_pattern = re.compile("\d+")
+            minutes_unit_pattern = re.compile("\w+")
+
+            minutes = int(re.findall(minute_number_pattern,custom_info["Running time"])[0])
+            minute_unit = re.findall(minutes_unit_pattern,custom_info["Running time"])[-1]
+            movie_info[f"Running time({minute_unit})"] = minutes
+        except Exception as e:
+            pass
+        else: 
+            del custom_info["Running time"]
         
-        return cast_urls
+
+        # getting the gross of the movie
+        try: 
+            currency, gross = money_string_to_int(custom_info,"Box office")
+            movie_info[f"Gross({currency})"] = gross
+        except Exception as e:
+            pass
+        else:
+            del custom_info["Box office"]
+
+        try: 
+            currency, budget = money_string_to_int(custom_info,"Budget")
+            movie_info[f"Budget({currency})"] = budget
+        except Exception as e:
+            pass
+        else:
+            del custom_info["Budget"]
+
+        if len(custom_info) < 2:
+            return
+
+        return custom_info
 
 
 
@@ -697,3 +803,4 @@ class WikiScraper:
 
 # end = perf_counter()
 # print(f"In total it took {round(end-start,2)} second(s)")
+
